"use strict";
const tslib_1 = require("tslib");
const semver = tslib_1.__importStar(require("semver"));
const path_1 = require("path");
const command_1 = require("@oclif/command");
const cli_ux_1 = tslib_1.__importDefault(require("cli-ux"));
const config_1 = require("./utils/config");
const chain_1 = require("./utils/chain");
const storage_1 = require("./utils/storage");
const genesis_block_1 = require("./utils/genesis_block");
const compatibleVersions = '>=2.1.4 <=2.1.6';
class LiskMigrator extends command_1.Command {
    async run() {
        var _a, _b;
        const { flags } = this.parse(LiskMigrator);
        const liskCorePath = (_a = flags['lisk-core-path']) !== null && _a !== void 0 ? _a : process.cwd();
        const outputPath = (_b = flags.output) !== null && _b !== void 0 ? _b : path_1.join(process.cwd(), 'genesis_block.json');
        const snapshotHeight = flags['snapshot-height'];
        const waitThreshold = process.env.NODE_ENV === 'test' ? flags['wait-threshold'] : 201;
        const config = await config_1.getConfig(liskCorePath);
        cli_ux_1.default.action.start('Verifying Lisk-Core version');
        const liskCoreVersion = semver.coerce(config.app.version);
        if (!liskCoreVersion) {
            this.error('Unable to detect the lisk-core version.');
        }
        if (!semver.satisfies(liskCoreVersion, compatibleVersions)) {
            this.error(`Lisk-Migrator utility is not compatible for lisk-core version ${liskCoreVersion.version}. Compatible versions range is: ${compatibleVersions}`);
        }
        cli_ux_1.default.action.stop(`${liskCoreVersion.version} detected`);
        const storageConfig = config.components.storage;
        cli_ux_1.default.action.start(`Verifying connection to database "${storageConfig.database}"`);
        const db = storage_1.createDb(storageConfig);
        await storage_1.verifyConnection(db);
        cli_ux_1.default.action.stop();
        await chain_1.observeChainHeight({
            label: 'Waiting for snapshot height',
            db,
            height: snapshotHeight,
            delay: 500,
        });
        cli_ux_1.default.action.start('Creating snapshot');
        const time = Date.now();
        await storage_1.createSnapshot(db);
        cli_ux_1.default.action.stop(`done in ${Date.now() - time}ms`);
        await chain_1.observeChainHeight({
            label: 'Waiting for threshold height',
            db,
            height: snapshotHeight + waitThreshold,
            delay: 500,
        });
        const genesisBlock = await genesis_block_1.createGenesisBlockFromStorage({
            db,
            snapshotHeight,
            epochTime: config.app.genesisConfig.EPOCH_TIME,
        });
        cli_ux_1.default.action.start('Exporting genesis block');
        genesis_block_1.writeGenesisBlock(genesisBlock, outputPath);
        cli_ux_1.default.action.stop();
        this.log(outputPath);
        db.$pool.end();
    }
}
LiskMigrator.description = 'Migrate Lisk Core to latest version';
LiskMigrator.flags = {
    version: command_1.flags.version({ char: 'v' }),
    help: command_1.flags.help({ char: 'h' }),
    output: command_1.flags.string({
        char: 'o',
        required: false,
        description: 'File path to write the genesis block json. If not provided, it will default to cwd/genesis_block.json.',
    }),
    'lisk-core-path': command_1.flags.string({
        char: 'p',
        required: false,
        description: 'Path where the lisk-core instance is running. Current directory will be considered the default if not provided.',
    }),
    'snapshot-height': command_1.flags.integer({
        char: 's',
        required: true,
        env: 'SNAPSHOT_HEIGHT',
        description: 'The height at which re-genesis block will be generated. Can be specified with SNAPSHOT_HEIGHT as well.',
    }),
    'wait-threshold': command_1.flags.integer({
        char: 'w',
        required: true,
        env: 'SNAPSHOT_WAIT_THRESHOLD',
        description: 'Blocks to wait before creating a snapshot. Applies only if NODE_ENV=test otherwise 201 value be used.',
        default: 201,
    }),
};
module.exports = LiskMigrator;
//# sourceMappingURL=index.js.map